package testcase;

import static org.junit.Assert.fail;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;

import javax.sql.DataSource;

import org.h2.jdbcx.JdbcDataSource;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.teiid.adminapi.impl.ModelMetaData;
import org.teiid.jdbc.RequestOptions;
import org.teiid.jdbc.StatementCallback;
import org.teiid.jdbc.TeiidDriver;
import org.teiid.jdbc.TeiidStatement;
import org.teiid.runtime.EmbeddedConfiguration;
import org.teiid.runtime.EmbeddedServer;
import org.teiid.translator.TranslatorException;
import org.teiid.translator.jdbc.h2.H2ExecutionFactory;

import com.google.common.eventbus.DeadEvent;
import com.google.common.eventbus.Subscribe;

public class QueryPlanTestCase
{

    private static EmbeddedServer                                       server;

    private static EmbeddedServer.ConnectionFactoryProvider<DataSource> jdbcProvider;

    private static JdbcDataSource                                       h2ds;

    @BeforeClass
    public static void setUp() throws Exception
    {
        initH2Database();
        final EmbeddedConfiguration ec = new EmbeddedConfiguration();
        ec.setUserRequestSourceConcurrency(1);
        ec.setUseDisk(true);
        server = new EmbeddedServer();
        server.start(ec);
        server.addConnectionFactoryProvider("source-jdbc", jdbcProvider);

        server.addTranslator(new H2ExecutionFactory());
        buildDeployVDB(server);
        System.out.println("TrafficLightContinuousQueries.setUp()");

    }

    @AfterClass
    public static void tearDown() throws Exception
    {

        server.stop();
    }

    @SuppressWarnings("unchecked")
    @Test
    public void testSimpleQuery() throws Exception
    {
        final String sql = "Select * From latest_health_states ";

        System.out.println("TrafficLightContinuousQueries.testTrafficLights()");

        final TeiidDriver td = server.getDriver();
        final Connection c = td.connect("jdbc:teiid:Chorus", null);
        final Statement stmt = c.createStatement();
        stmt.execute("SET NOEXEC ON");
        stmt.execute(sql);
        final TeiidStatement tstatement = stmt.unwrap(TeiidStatement.class);

        System.out.println("########## this " + this);

        tstatement.submitExecute(sql, new StatementCallback()
        {

            @Override
            public void onRow(final Statement s, final ResultSet rs) throws Exception
            {
                System.out.println(rs.getObject(1));
            }

            @Override
            public void onException(final Statement s, final Exception e) throws Exception
            {
                e.printStackTrace();
            }

            @Override
            public void onComplete(final Statement s) throws Exception
            {
                System.out.println("Complete");
            }
        },
                                 new RequestOptions().continuous(true));

        final org.teiid.client.plan.PlanNode queryPlan = tstatement.getPlanDescription();
        System.out.println("Plan: " + queryPlan);

        Thread.sleep(500000000);
        tstatement.cancel();

    }

    private static void initH2Database() throws Exception
    {
        h2ds = new JdbcDataSource();
        h2ds.setURL("jdbc:h2:mem/testInsert");

        jdbcProvider = new EmbeddedServer.ConnectionFactoryProvider<DataSource>()
        {
            @Override
            public DataSource getConnectionFactory() throws TranslatorException
            {
                return QueryPlanTestCase.h2ds;
            }
        };
        final Connection conn = jdbcProvider.getConnectionFactory()
                                            .getConnection();

        final Statement stmt = conn.createStatement();

        stmt.execute("DROP table IF EXISTS latest_health_states");
        stmt.execute("CREATE TABLE latest_health_states(HEALTHTIME TIMESTAMP, HEALTHSTATEFKEY INTEGER,POLICYFKEYXML VARCHAR, "
                     + "OBJFKEYXML VARCHAR )");

    }

    private static void buildDeployVDB(final EmbeddedServer teiidServer) throws Exception
    {

        final ModelMetaData h2Model = new ModelMetaData();

        h2Model.setName("h2Translator");
        h2Model.setSchemaSourceType("native");
        h2Model.addSourceMapping("h2Translator", "h2", "source-jdbc");

        teiidServer.deployVDB("Chorus", h2Model);

        System.out.println("TrafficLightContinuousQueries.buildDeployVDB()");
    }

    @Subscribe
    public void handleDeadEvent(final DeadEvent deadEvent)
    {
        fail("dead event: " + deadEvent);
    }

}
